@@ -85,6 +85,7 @@ XineEngine::XineEngine()
     #endif
     addPluginProperty("HasCDDA", "true"); // new property
     debug() << "hello" << endl;
+
 }
 
 XineEngine::~XineEngine()
@@ -94,14 +95,15 @@ XineEngine::~XineEngine()
         m_stopFader = true;
         s_fader->resume(); // safety call if the engine is in the pause state
         s_fader->wait();
-        delete s_fader;
-    }
-    if( s_outfader ) {
-        s_outfader->wait();
-        delete s_outfader;
     }
 
-    fadeOut( true ); // true == exiting
+    delete s_fader;
+    delete s_outfader;
+
+    if( AmarokConfig::fadeoutOnExit() ) {
+        bool terminateFader = false;
+        fadeOut( AmarokConfig::fadeoutLength(), &terminateFader, true ); // true == exiting
+    }
 
     if( m_xine )       xine_config_save( m_xine, configPath() );
 
@@ -190,7 +192,7 @@ XineEngine::makeNewStream()
     if ( xine_check_version(1,1,1) && !(m_xfadeLength > 0) ) {
         // enable gapless playback
         debug() << "gapless playback enabled." << endl;
-        xine_set_param(m_stream, XINE_PARAM_EARLY_FINISHED_EVENT, 1 );
+        //xine_set_param(m_stream, XINE_PARAM_EARLY_FINISHED_EVENT, 1 );
     }
 #endif
    return true;
@@ -216,7 +218,14 @@ XineEngine::load( const KURL &url, bool isStream )
 
     Engine::Base::load( url, isStream );
 
+    if( s_outfader ) {
+        s_outfader->finish();
+        delete s_outfader;
+    }
+
     if( m_xfadeLength > 0 && xine_get_status( m_stream ) == XINE_STATUS_PLAY &&
+         url.isLocalFile() &&
+         xine_get_param( m_stream, XINE_PARAM_SPEED ) != XINE_SPEED_PAUSE &&
         ( m_xfadeNextTrack || //set by engine controller when switching tracks automatically
          (uint) AmarokConfig::crossfadeType() == 0 ||  //crossfade always
          (uint) AmarokConfig::crossfadeType() == 2 ) ) //crossfade when switching tracks manually
@@ -225,7 +234,7 @@ XineEngine::load( const KURL &url, bool isStream )
        // Stop a probably running fader
        if( s_fader ) {
            m_stopFader = true;
-           s_fader->terminate(); // kills a running fader
+           s_fader->finish(); // makes the fader stop abruptly
            delete s_fader;
        }
        s_fader = new Fader( this, m_xfadeLength );
@@ -251,6 +260,8 @@ XineEngine::load( const KURL &url, bool isStream )
       xine_post_wire( source, target );
       #endif
 
+      playlistChanged();
+
       return true;
    }
    else
@@ -364,17 +375,24 @@ XineEngine::stop()
 
     if ( !m_stream )
        return;
-    if( !m_fadeOutRunning || state() == Engine::Paused )
+
+    if( AmarokConfig::fadeout() && !m_fadeOutRunning || state() == Engine::Paused )
     {
-        s_outfader = new OutFader( this, true, true );
+        s_outfader = new OutFader( this, AmarokConfig::fadeoutLength() );
         s_outfader->start();
         ::usleep( 100 ); //to be sure engine state won't be changed before it is checked in fadeOut()
         m_url = KURL(); //to ensure we return Empty from state()
 
         std::fill( m_scope.begin(), m_scope.end(), 0 );
-
-        emit stateChanged( Engine::Empty );
     }
+    else if( !m_fadeOutRunning )
+    {
+        xine_stop( m_stream );
+        xine_close( m_stream );
+        xine_set_param( m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
+    }
+
+    emit stateChanged( Engine::Empty );
 }
 
 void
@@ -388,12 +406,21 @@ XineEngine::pause()
         if( s_fader && s_fader->running() )
             s_fader->pause();
 
-        xine_set_param( m_stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE ); 
+        xine_set_param( m_stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE );
         xine_set_param( m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
         emit stateChanged( Engine::Paused );
 
-    } else {
+    }
+}
 
+void
+XineEngine::unpause()
+{
+    if ( !m_stream )
+        return;
+    
+    if( xine_get_param( m_stream, XINE_PARAM_SPEED ) == XINE_SPEED_PAUSE )
+    {
         if( s_fader && s_fader->running() )
             s_fader->resume();
 
@@ -420,16 +447,22 @@ XineEngine::state() const
 uint
 XineEngine::position() const
 {
-    if ( !m_stream )
+    if ( state() == Engine::Empty )
        return 0;
 
     int pos;
     int time = 0;
     int length;
 
-    xine_get_pos_length( m_stream, &pos, &time, &length );
-
-
+    // Workaround for problems when you seek too quickly, see BUG 99808
+    int tmp = 0, i = 0;
+    while( ++i < 4 )
+    {
+        xine_get_pos_length( m_stream, &pos, &time, &length );
+        if( time > tmp ) break;
+        usleep( 100000 );
+    }
+			
     // Here we check for new metadata periodically, because xine does not emit an event
     // in all cases (e.g. with ogg streams). See BUG 122505
     if ( state() != Engine::Idle && state() != Engine::Empty )
@@ -465,7 +498,9 @@ XineEngine::length() const
         int length = 0;
 
         xine_get_pos_length( m_stream, &pos, &time, &length );
-
+        if( length < 0 )
+            length=0;
+	    	
         return length;
     }
 }
@@ -495,7 +530,7 @@ XineEngine::setVolumeSW( uint vol )
 }
 
 void
-XineEngine::fadeOut( bool exiting )
+XineEngine::fadeOut( uint fadeLength, bool* terminate, bool exiting )
 {
     if( m_fadeOutRunning ) //Let us not start another fadeout...
         return;
@@ -505,10 +540,9 @@ XineEngine::fadeOut( bool exiting )
     const float originalVol = Engine::Base::makeVolumeLogarithmic( m_volume ) * m_preamp;
 
     // On shutdown, limit fadeout to 3 secs max, so that we don't risk getting killed
-    const int length = exiting ? QMIN( m_xfadeLength, 3000 ) : m_xfadeLength;
+    const int length = exiting ? QMIN( fadeLength, 3000 ) : fadeLength;
 
-    // NOTE The fadeout gets stuck when the EQ is active, so we skip it then
-    if( length > 0 && !m_equalizerEnabled && isPlaying )
+    if( length > 0 && isPlaying )
     {
         // fader-class doesn't work in this spot as is, so some parts need to be copied here... (ugly)
         uint stepsCount = length < 1000 ? length / 10 : 100;
@@ -520,6 +554,8 @@ XineEngine::fadeOut( bool exiting )
         float mix = 0.0;
         while ( mix < 1.0 )
         {
+            if( *terminate ) break;
+
             ::usleep( stepSizeUs );
             float vol = Engine::Base::makeVolumeLogarithmic( m_volume ) * m_preamp;
             float mix = (float)t.elapsed() / (float)length;
@@ -550,23 +586,22 @@ XineEngine::setEqualizerEnabled( bool enable )
     if( !enable ) {
         QValueList<int> gains;
         for( uint x = 0; x < 10; x++ )
-            gains << -102; // sets eq gains to zero.
+            gains << -101; // sets eq gains to zero.
 
         setEqualizerParameters( 0, gains );
    }
 }
 
 /*
- sets the eq params for xine engine - have to rescale eq params to fitting range (adapted from kaffeine)
- sideeffect: volume decreases with enabled eq, don't know why, occurs also in kaffeine
+ sets the eq params for xine engine - have to rescale eq params to fitting range (adapted from kaffeine and xfmedia)
 
  preamp
    pre: (-100..100)
-   post: (0.2..1.8) - this is not really a preamp but xine offers no preamp so we make a postamp
+   post: (0.1..1.9) - this is not really a preamp but we use the xine preamp parameter for our normal volume. so we make a postamp.
 
  gains
    pre: (-100..100)
-   post: (1..100) - range adapted from kaffeine (1 = down, 50 = middle, 100 = up, 0 = off[!!!])
+   post: (1..200) - (1 = down, 100 = middle, 200 = up, 0 = off)
  */
 void
 XineEngine::setEqualizerParameters( int preamp, const QValueList<int> &gains )
@@ -578,18 +613,18 @@ XineEngine::setEqualizerParameters( int preamp, const QValueList<int> &gains )
     m_intPreamp = preamp;
     QValueList<int>::ConstIterator it = gains.begin();
 
-    xine_set_param( m_stream, XINE_PARAM_EQ_30HZ,    int( (*it  )*0.49 + 50 ) );
-    xine_set_param( m_stream, XINE_PARAM_EQ_60HZ,    int( (*++it)*0.49 + 50 ) );
-    xine_set_param( m_stream, XINE_PARAM_EQ_125HZ,   int( (*++it)*0.49 + 50 ) );
-    xine_set_param( m_stream, XINE_PARAM_EQ_250HZ,   int( (*++it)*0.49 + 50 ) );
-    xine_set_param( m_stream, XINE_PARAM_EQ_500HZ,   int( (*++it)*0.49 + 50 ) );
-    xine_set_param( m_stream, XINE_PARAM_EQ_1000HZ,  int( (*++it)*0.49 + 50 ) );
-    xine_set_param( m_stream, XINE_PARAM_EQ_2000HZ,  int( (*++it)*0.49 + 50 ) );
-    xine_set_param( m_stream, XINE_PARAM_EQ_4000HZ,  int( (*++it)*0.49 + 50 ) );
-    xine_set_param( m_stream, XINE_PARAM_EQ_8000HZ,  int( (*++it)*0.49 + 50 ) );
-    xine_set_param( m_stream, XINE_PARAM_EQ_16000HZ, int( (*++it)*0.49 + 50 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_30HZ,    int( (*it  )*0.995 + 100 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_60HZ,    int( (*++it)*0.995 + 100 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_125HZ,   int( (*++it)*0.995 + 100 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_250HZ,   int( (*++it)*0.995 + 100 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_500HZ,   int( (*++it)*0.995 + 100 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_1000HZ,  int( (*++it)*0.995 + 100 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_2000HZ,  int( (*++it)*0.995 + 100 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_4000HZ,  int( (*++it)*0.995 + 100 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_8000HZ,  int( (*++it)*0.995 + 100 ) );
+    xine_set_param( m_stream, XINE_PARAM_EQ_16000HZ, int( (*++it)*0.995 + 100 ) );
 
-    m_preamp = ( preamp - 0.2 * preamp + 100 ) / 100.0;
+    m_preamp = ( preamp - 0.1 * preamp + 100 ) / 100.0;
     setVolume( m_volume );
 }
 
@@ -821,6 +856,28 @@ void XineEngine::configChanged()
     }
 }
 
+//SLOT
+void
+XineEngine::playlistChanged()
+{
+    #ifdef XINE_PARAM_EARLY_FINISHED_EVENT
+    #ifdef XINE_PARAM_GAPLESS_SWITCH
+    if ( xine_check_version(1,1,1) && !(m_xfadeLength > 0) 
+         && m_url.isLocalFile() && Playlist::instance()->isTrackAfter() )
+    {
+        xine_set_param(m_stream, XINE_PARAM_EARLY_FINISHED_EVENT, 1 );
+        debug() << "XINE_PARAM_EARLY_FINISHED_EVENT enabled" << endl;
+    }
+    else
+    {
+        //we don't want an early finish event if there is no track after the current one
+        xine_set_param(m_stream, XINE_PARAM_EARLY_FINISHED_EVENT, 0 );
+        debug() << "XINE_PARAM_EARLY_FINISHED_EVENT disabled" << endl;
+    }
+    #endif
+    #endif
+}
+
 static time_t last_error_time = 0; // hysteresis on xine errors
 static int    last_error = XINE_MSG_NO_ERROR;
 
@@ -1008,162 +1065,6 @@ XineEngine::fetchMetaData() const
     return bundle;
 }
 
-
-//////////////////
-/// class Fader
-//////////////////
-
-Fader::Fader( XineEngine *engine, uint fadeMs )
-   : QObject( engine )
-   , QThread()
-   , m_engine( engine )
-   , m_xine( engine->m_xine )
-   , m_decrease( engine->m_stream )
-   , m_increase( 0 )
-   , m_port( engine->m_audioPort )
-   , m_post( engine->m_post )
-   , m_fadeLength( fadeMs )
-   , m_paused( false )
-{
-    if( engine->makeNewStream() )
-    {
-        m_increase = engine->m_stream;
-
-        xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, 0 );
-    }
-    else {
-        s_fader = 0;
-        deleteLater();
-    }
-}
-
-Fader::~Fader()
-{
-     wait();
-
-     DEBUG_FUNC_INFO
-
-     xine_close( m_decrease );
-     xine_dispose( m_decrease );
-     xine_close_audio_driver( m_xine, m_port );
-     if( m_post ) xine_post_dispose( m_xine, m_post );
-
-     if( !m_engine->m_stopFader )
-         m_engine->setVolume( m_engine->volume() );
-
-     m_engine->m_stopFader = false;
-     s_fader = 0;
-}
-
-void
-Fader::run()
-{
-    // do a volume change in 100 steps (or every 10ms)
-    uint stepsCount = m_fadeLength < 1000 ? m_fadeLength / 10 : 100;
-    uint stepSizeUs = (int)( 1000.0 * (float)m_fadeLength / (float)stepsCount );
-
-    float mix = 0.0;
-    float elapsedUs = 0.0;
-    while ( mix < 1.0 )
-    {
-        // sleep a constant amount of time
-        QThread::usleep( stepSizeUs );
-
-        if ( m_paused )
-        	continue;
-
-        elapsedUs += stepSizeUs;
-
-        // get volume (amarok main * equalizer preamp)
-        float vol = Engine::Base::makeVolumeLogarithmic( m_engine->m_volume ) * m_engine->m_preamp;
-
-        // compute the mix factor as the percentage of time spent since fade begun
-        float mix = (elapsedUs / 1000.0) / (float)m_fadeLength;
-        if ( mix > 1.0 )
-        {
-            if ( m_increase )
-                xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)vol );
-            break;
-        }
-
-        // change volume of streams (using dj-like cross-fade profile)
-        if ( m_decrease )
-        {
-            //xine_set_param( m_decrease, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)(vol * (1.0 - mix)) );  // linear
-            float v = 4.0 * (1.0 - mix) / 3.0;
-            xine_set_param( m_decrease, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)( v < 1.0 ? vol * v : vol ) );
-        }
-        if ( m_increase )
-        {
-            //xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)(vol * mix) );  //linear
-            float v = 4.0 * mix / 3.0;
-            xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)( v < 1.0 ? vol * v : vol ) );
-        }
-    }
-
-    //stop using cpu!
-    xine_stop( m_decrease );
-
-    deleteLater();
-}
-
-void
-Fader::pause()
-{
-	m_paused = true;
-}
-
-void
-Fader::resume()
-{
-	m_paused = false;
-}
-
-
-//////////////////
-/// class OutFader
-//////////////////
-
-OutFader::OutFader( XineEngine *engine, bool stop, bool force )
-   : QObject( engine )
-   , QThread()
-   , m_engine( engine )
-   , m_stop( stop )
-   , m_force( force )
-{
-}
-
-OutFader::~OutFader()
-{
-     wait();
-
-     DEBUG_FUNC_INFO
-
-     s_outfader = 0;
-}
-
-void
-OutFader::run()
-{
-    m_engine->fadeOut();
-    if( m_engine->m_fadeOutRunning == false || m_force )
-    {
-        if( m_stop )
-        {
-            xine_stop( m_engine->m_stream );
-            xine_close( m_engine->m_stream );
-            xine_set_param( m_engine->m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
-        }
-        else
-        {
-            xine_set_param( m_engine->m_stream, XINE_PARAM_SPEED, XINE_SPEED_PAUSE );
-            xine_set_param( m_engine->m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
-        }
-    }
-    QThread::sleep( 3 );
-    deleteLater();
-}
-
 bool XineEngine::metaDataForUrl(const KURL &url, Engine::SimpleMetaBundle &b)
 {
     bool result = false;
@@ -1230,7 +1131,10 @@ bool XineEngine::getAudioCDContents(const QString &device, KURL::List &urls)
     if (!device.isNull()) {
         debug() << "xine-engine setting CD Device to: " << device << endl;
         xine_cfg_entry_t config;
-        xine_config_lookup_entry(m_xine, "input.cdda_device", &config);
+        if (!xine_config_lookup_entry(m_xine, "input.cdda_device", &config)) {
+	    emit statusText(i18n("Failed CD device lookup in xine engine"));
+	    return false;
+	}
         config.str_value = (char *)device.latin1();
         xine_config_update_entry(m_xine, &config);
     }
@@ -1255,4 +1159,172 @@ bool XineEngine::flushBuffer()
     return false;
 }
 
+//////////////////////////////////////////////////////////////////////////////
+/// class Fader
+//////////////////////////////////////////////////////////////////////////////
+
+Fader::Fader( XineEngine *engine, uint fadeMs )
+   : QObject( engine )
+   , QThread()
+   , m_engine( engine )
+   , m_xine( engine->m_xine )
+   , m_decrease( engine->m_stream )
+   , m_increase( 0 )
+   , m_port( engine->m_audioPort )
+   , m_post( engine->m_post )
+   , m_fadeLength( fadeMs )
+   , m_paused( false )
+   , m_terminated( false )
+{
+    DEBUG_BLOCK
+
+    if( engine->makeNewStream() )
+    {
+        m_increase = engine->m_stream;
+
+        xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, 0 );
+    }
+    else {
+        s_fader = 0;
+        deleteLater();
+    }
+}
+
+Fader::~Fader()
+{
+     DEBUG_BLOCK
+
+     wait();
+
+     xine_close( m_decrease );
+     xine_dispose( m_decrease );
+     xine_close_audio_driver( m_xine, m_port );
+     if( m_post ) xine_post_dispose( m_xine, m_post );
+
+     if( !m_engine->m_stopFader )
+         m_engine->setVolume( m_engine->volume() );
+
+     m_engine->m_stopFader = false;
+     s_fader = 0;
+}
+
+void
+Fader::run()
+{
+    DEBUG_BLOCK
+
+    // do a volume change in 100 steps (or every 10ms)
+    uint stepsCount = m_fadeLength < 1000 ? m_fadeLength / 10 : 100;
+    uint stepSizeUs = (int)( 1000.0 * (float)m_fadeLength / (float)stepsCount );
+
+    float mix = 0.0;
+    float elapsedUs = 0.0;
+    while ( mix < 1.0 )
+    {
+        if ( m_terminated )
+            break;
+        // sleep a constant amount of time
+        QThread::usleep( stepSizeUs );
+
+        if ( m_paused )
+        	continue;
+
+        elapsedUs += stepSizeUs;
+
+        // get volume (amarok main * equalizer preamp)
+        float vol = Engine::Base::makeVolumeLogarithmic( m_engine->m_volume ) * m_engine->m_preamp;
+
+        // compute the mix factor as the percentage of time spent since fade begun
+        float mix = (elapsedUs / 1000.0) / (float)m_fadeLength;
+        if ( mix > 1.0 )
+        {
+            if ( m_increase )
+                xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)vol );
+            break;
+        }
+
+        // change volume of streams (using dj-like cross-fade profile)
+        if ( m_decrease )
+        {
+            //xine_set_param( m_decrease, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)(vol * (1.0 - mix)) );  // linear
+            float v = 4.0 * (1.0 - mix) / 3.0;
+            xine_set_param( m_decrease, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)( v < 1.0 ? vol * v : vol ) );
+        }
+        if ( m_increase )
+        {
+            //xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)(vol * mix) );  //linear
+            float v = 4.0 * mix / 3.0;
+            xine_set_param( m_increase, XINE_PARAM_AUDIO_AMP_LEVEL, (uint)( v < 1.0 ? vol * v : vol ) );
+        }
+    }
+
+    //stop using cpu!
+    xine_stop( m_decrease );
+
+    deleteLater();
+}
+
+void
+Fader::pause()
+{
+	m_paused = true;
+}
+
+void
+Fader::resume()
+{
+	m_paused = false;
+}
+
+void
+Fader::finish()
+{
+    DEBUG_BLOCK
+	m_terminated = true;
+}
+
+//////////////////////////////////////////////////////////////////////////////
+/// class OutFader
+//////////////////////////////////////////////////////////////////////////////
+
+OutFader::OutFader( XineEngine *engine, uint fadeLength )
+   : QObject( engine )
+   , QThread()
+   , m_engine( engine )
+   , m_terminated( false )
+   , m_fadeLength( fadeLength )
+{
+    DEBUG_BLOCK
+}
+
+OutFader::~OutFader()
+{
+     DEBUG_BLOCK
+
+     wait();
+
+     s_outfader = 0;
+}
+
+void
+OutFader::run()
+{
+    DEBUG_BLOCK
+
+    m_engine->fadeOut( m_fadeLength, &m_terminated );
+
+    xine_stop( m_engine->m_stream );
+    xine_close( m_engine->m_stream );
+    xine_set_param( m_engine->m_stream, XINE_PARAM_AUDIO_CLOSE_DEVICE, 1);
+
+    deleteLater();
+}
+
+void
+OutFader::finish()
+{
+    DEBUG_BLOCK
+	m_terminated = true;
+}
+
 #include "xine-engine.moc"
