@@ -9,14 +9,17 @@
 ** These will automatically be called by the form's constructor and
 ** destructor.
 *****************************************************************************/
+
+#include "amarok.h"
 #include "config.h"
 #include "directorylist.h"
+
 #include <kapplication.h>
 #include <kconfig.h>
 #include <klocale.h>
 #include <qpushbutton.h>
 
-namespace amaroK
+namespace Amarok
 {
     extern QPixmap getPNG( const QString& );
     extern QPixmap getJPG( const QString& );
@@ -27,23 +30,11 @@ namespace amaroK
 void
 FirstRunWizard::init()
 {
-    using namespace amaroK;
+    using namespace Amarok;
 
     //aesthetics
-    cancelButton()->setFixedSize( cancelButton()->sizeHint() );
     helpButton()->hide();
 
-    //would be better as a KConfigXT key now
-    if ( config( "General" )->readEntry( "XMLFile", QString::null ) != "amarokui.rc" ) {
-        option_xmms->setChecked( true );
-        picture_modePreview->setPixmap( getPNG( "wizard_xmms" ) );
-    }
-    else {
-        option_compact->setChecked( true );
-        picture_modePreview->setPixmap( getPNG( "wizard_compact" ) );
-    }
-    picture_modePreview->adjustSize();
-
     picture1->setPixmap( getJPG( "amarok_rocks" ) );
     picture4->setPixmap( *picture1->pixmap() );
 
@@ -75,32 +66,14 @@ FirstRunWizard::invokeHandbook() //SLOT
     kapp->invokeHelp( QString::null, QString::null, 0 );
 }
 
-inline void
-FirstRunWizard::xmmsModeToggled( bool on ) //SLOT
-{
-    using namespace amaroK;
-
-    if ( on )
-        picture_modePreview->setPixmap( getPNG( "wizard_xmms" ) );
-    else
-        picture_modePreview->setPixmap( getPNG( "wizard_compact" ) );
-}
-
-FirstRunWizard::Interface
-FirstRunWizard::interface()
-{
-    return option_xmms->isChecked() ? XMMS : Compact;
-}
-
 void
 FirstRunWizard::writeCollectionConfig()
 {
     m_folderSetup->writeConfig();
 }
 
-
 void
 FirstRunWizard::openLink(const QString& s)
 {
-    kapp->invokeBrowser(s);
+    Amarok::invokeBrowser(s);
 }
