@@ -223,7 +223,7 @@
 // then decides to clear the playlist again.  The jobEvent() signal
 // passes the URL of the job that was completed.
 //
-// The analyzer is actually run using a KProcess.  ThreadWeaver::Job
+// The analyzer is actually run using a KProcess.  ThreadManager::Job
 // is not a good solution, since we need more flexibility in the
 // queuing process, and in addition, KProcess'es must be started from
 // the GUI thread!
@@ -1222,7 +1222,7 @@ Moodbar::readFile( void )
             val        = 250;
           }
 
-        debug() << "ReadMood: Appling filter t=" << threshold
+        debug() << "ReadMood: Applying filter t=" << threshold
                 << ", rS=" << rangeStart << ", rD=" << rangeDelta
                 << ", s=" << sat << "%, v=" << val << "%" << endl;
 
@@ -1321,7 +1321,7 @@ Moodbar::moodFilename( const KURL &url, bool withMusic )
         path.truncate(path.findRev('.'));
 
         if (path.isEmpty())  // Weird...
-          return QString::null;
+          return QString();
 
         path += ".mood";
         int slash = path.findRev('/') + 1;
@@ -1341,7 +1341,7 @@ Moodbar::moodFilename( const KURL &url, bool withMusic )
         path.truncate(path.findRev('.'));
 
         if (path.isEmpty())  // Weird...
-          return QString::null;
+          return QString();
 
         path = QString::number( deviceid ) + ','
           + path.replace('/', ',') + ".mood";
